import * as fs from "fs";
import * as VAPI from "vapi";
import { WaveFile } from "./biw-utils.js";
export declare class AudioStreamDistributor<T> {
    private cbs;
    constructor();
    add(cb: (data: T, seqnr: number, ts: number) => void): void;
    remove(cb: (data: T, seqnr: number, ts: number) => void): void;
    process(data: T, seqnr: number, ts: number): void;
    chain(): (data: T, seqnr: number, ts: number) => void;
}
export declare class AudioStreamReceiver {
    readonly num_channels: VAPI.Audio.Channelcount;
    readonly format: VAPI.Audio.Format;
    private m_socket;
    private m_lastSeqNr;
    private m_lastRtpTS;
    private m_rtpTSErrors;
    private m_seqNrErrors;
    on_data?: (data: WaveFile, seqnr: number, ts: number) => void;
    on_error?: (err: any) => void;
    constructor(num_channels: VAPI.Audio.Channelcount, format: VAPI.Audio.Format);
    get rtpTSErrors(): number;
    get seqNrErrors(): number;
    start(addrs: {
        src: string;
        dst: string;
        port: number;
    }): Promise<void>;
    private handle_rtp_header;
    stop(): void;
}
export declare class AudioStreamWriter {
    private writer;
    private hdr;
    constructor(path: fs.PathLike);
    get num_samples(): number;
    process(data: WaveFile): void;
    write_header(): void;
    chain(): (data: WaveFile) => void;
    close(): void;
}
export declare class AudioNormalizedRingBuffer {
    private buf;
    private pos;
    on_data?: (samples: Float32Array[], wave: WaveFile) => void;
    constructor(pars: {
        num_channels: number;
        num_samples: number;
    });
    process(data: WaveFile): void;
    reset(): void;
    chain(): (data: WaveFile, _seqnr: number, _rtpts: number) => void;
    chain_reset_on_lost_packet(lost_cb?: (seqnr: number, expected: number) => void): (data: WaveFile, seqnr: number, _rtpts: number) => void;
}
export interface AudioFFTParams {
    num_channels: number;
    num_samples: number;
    sample_rate: number;
    window: "Hann";
}
export interface AudioFFTPeak {
    n: number;
    freq: number;
    power: number;
    db: number;
}
export interface AudioFFTResultChannel {
    get_power: (n: number) => number;
    get_db: (n: number) => number;
    find_peaks: (threshold_dB?: number) => AudioFFTPeak[];
}
export interface AudioFFTResult {
    length: number;
    channels: number;
    frequency(n: number): number;
    channel(ch: number): AudioFFTResultChannel;
}
export declare class AudioFFT {
    private fft;
    private out;
    private tmp;
    private window;
    private sample_rate;
    readonly scale: number;
    on_data?: (res: AudioFFTResult) => void;
    constructor(pars: AudioFFTParams);
    process(data: Float32Array[]): void;
    chain(): (data: Float32Array[]) => void;
}
